# -*- coding: iso-8859-1 -*-
"""
    MoinMoin - SharedUpdate Action

    @copyright: 2012 by Paul Boddie <paul@boddie.org.uk>
    @license: GNU GPL (v2 or later), see COPYING.txt for details.
"""

from MoinMoin.action import ActionBase
from MoinMoin import wikiutil
from MoinShare import *

Dependencies = ['pages']

# Action class and supporting functions.

class SharedUpdate(ActionBase, ActionSupport):

    "An update retrieval dialogue requesting various parameters."

    def get_form_html(self, buttons_html):
        _ = self._
        request = self.request
        form = self.get_form()
        page = self.page

        pagename = form.get("pagename", [page.page_name])[0]
        fragment = form.get("fragment", [""])[0]
        mimetype = form.get("type", [""])[0]

        d = {
            "buttons_html"          : buttons_html,
            "pagename_label"        : escape(_("Page name for updates")),
            "pagename"              : escattr(pagename),
            "fragment_label"        : escape(_("Fragment to download")),
            "fragment"              : escattr(fragment),
            "mimetype_label"        : escape(_("Content/media type")),
            "mimetype"              : escattr(mimetype),
            }

        return '''
<table>
    <tr>
        <td class="label"><label>%(pagename_label)s</label></td>
        <td class="content">
            <input name="pagename" type="text" size="40" value="%(pagename)s" />
        </td>
    </tr>
    <tr>
        <td class="label"><label>%(fragment_label)s</label></td>
        <td class="content">
            <input name="fragment" type="text" size="40" value="%(fragment)s" />
        </td>
    </tr>
    <tr>
        <td class="label"><label>%(mimetype_label)s</label></td>
        <td class="content">
            <input name="type" type="text" size="40" value="%(mimetype)s" />
        </td>
    </tr>
    <tr>
        <td></td>
        <td class="buttons">
            %(buttons_html)s
        </td>
    </tr>
</table>
''' % d

    def do_action(self):

        "Write the syndication resource."

        _ = self._
        form = self.get_form()

        # If no fragment exists in the request, an error message is returned.

        fragment = form.get("fragment")

        if not fragment:
            return 0, _("No fragment to download specified.")

        write_resource(self.request)
        return 1, None

    def render_success(self, msg, msgtype=None):

        """
        Render neither 'msg' nor 'msgtype' since a resource has already been
        produced.
        NOTE: msgtype is optional because MoinMoin 1.5.x does not support it.
        """

        pass

def write_resource(request):

    """
    For the given 'request', write the requested fragment with the specified
    content/media type (or use content negotiation to choose a media type).
    """

    form = get_form(request)

    pagename = form.get("pagename", [request.page.page_name])[0]
    fragment = form.get("fragment", [None])[0]
    mimetype = form.get("type", [None])[0]

    page = Page(request, pagename)
    metadata = getMetadata(page)

    # Set the default error status.

    status = "404 Not Found"

    # Get the fragment regions for the page, trying to find the requested
    # fragment.

    for n, (format, attributes, body) in enumerate(getFragments(page.get_raw_body())):

        # Produce a fragment identifier.
        # NOTE: Choose a more robust identifier where none is explicitly given.

        region_fragment = attributes.get("fragment", str(n))

        # Check the region's fragment identifier.

        if fragment != region_fragment:
            continue

        # Check the mimetype availability.

        mimetypes = getOutputTypes(request, format)

        # Perform content negotiation if no mimetype was specified.

        if not mimetype:
            mimetypes = getPreferredOutputTypes(request, mimetypes)
            if mimetypes:
                mimetype = mimetypes[0]

        # Where no suitable mimetype is found, break out of the loop and return
        # an error.

        if not mimetypes or mimetype not in mimetypes:
            status = "406 Not Acceptable"
            break

        # Define headers.

        writeHeaders(request, mimetype, metadata)
        updated = getUpdatedTime(metadata)

        # Fragment output...

        parser_cls = getParserClass(request, format)
        parser = parser_cls(body, request)

        if hasattr(parser, "formatForOutputType"):
            parser.formatForOutputType(mimetype)
        else:
            fmt = request.html_formatter
            fmt.setPage(page)
            parser.format(fmt)

        return

    # Error output.

    writeHeaders(request, "text/plain", metadata, status)
    request.write("Sorry, but the specified fragment %s could not be provided as type %s." % (fragment, mimetype))

# Action function.

def execute(pagename, request):
    SharedUpdate(pagename, request).render()

# vim: tabstop=4 expandtab shiftwidth=4
